/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "main.h"
#include "syscfg.h"

#define DESKTOPFILE DATADIR "/applications/lconfeditor.desktop"

#ifdef EEE
#define ACPICONFFILE "/etc/default/eeepc-acpi-scripts"
#endif

enum init_ids {
    INIT_SCREEN = 0,
    INIT_PERSISTENT,
    INIT_LOCK
};

/*
 * generic functions
 */
static void
set (GObject *obj, gpointer value)
{
    g_object_set_data (obj, "ltopconf", value);
}

gpointer
get (GObject *obj)
{
    return g_object_get_data (obj, "ltopconf");
}

/*
 * per setting functions
 */
/* language */
void set_locale (const gchar *code)
{
    set (G_OBJECT (dialog->locale_entry), (gpointer) g_strdup (code));
    gtk_entry_set_text (GTK_ENTRY (dialog->locale_entry),
        get_locale_name (code));
}

const gchar *get_locale ()
{
    const gchar *tmp =  (const gchar *) get (G_OBJECT (dialog->locale_entry));
    return g_strconcat (" locale=", tmp, NULL);
}

const gchar *get_locale_value ()
{
    return  (const gchar *) get (G_OBJECT (dialog->locale_entry));
}

/* kbd */
void set_kbd_user_defined (gboolean b)
{
    g_object_set_data (G_OBJECT (dialog->kbd_entry), "user_defined",
        (gpointer) b);
}

gboolean get_kbd_user_defined ()
{
    return (gboolean) g_object_get_data (G_OBJECT (dialog->kbd_entry), "user_defined");
}

void set_kbd (const gchar *code)
{
    set (G_OBJECT (dialog->kbd_entry), (gpointer) g_strdup (code));
    gtk_entry_set_text (GTK_ENTRY (dialog->kbd_entry),
        (const char *) (get_kbd_name (code)));
}

const gchar *get_kbd ()
{
    const gchar *code = (const gchar *) get (G_OBJECT (dialog->kbd_entry));
    gchar *tmp = g_strconcat (" console-setup/layoutcode=", code,
                              " xkbdlayout=", code, NULL);
    const gchar *ret = g_strdup (tmp);
    g_free (tmp);
    return ret;
}

const gchar *get_kbd_value ()
{
    return (const gchar *) get (G_OBJECT (dialog->kbd_entry));
}

static const gchar *get_init_kbd_value ()
{
    return (const gchar *) g_object_get_data (G_OBJECT (dialog->dialog), "init_kbd");
}

void restore_kbd_config ()
{
    const gchar *tmp = get_init_kbd_value ();
    apply_kbd_config (tmp);
}

/* timezone */
void set_tz_user_defined (gboolean b)
{
    g_object_set_data (G_OBJECT (dialog->tz_entry), "user_defined",
        (gpointer) b);
}

gboolean get_tz_user_defined ()
{
    return (gboolean) g_object_get_data (G_OBJECT (dialog->tz_entry), "user_defined");
}

void set_tz (const gchar *code)
{
    set (G_OBJECT (dialog->tz_entry), (gpointer) g_strdup (code));
    gtk_entry_set_text (GTK_ENTRY (dialog->tz_entry),
        code);
}

const gchar *get_tz ()
{
    const gchar *tmp = (const gchar *) get (G_OBJECT (dialog->tz_entry));
    return g_strconcat (" timezone=", tmp, NULL);
}

const gchar *get_tz_value ()
{
    return (const gchar *) get (G_OBJECT (dialog->tz_entry));
}

/* screen */
void set_screensize (const gchar *code)
{
    if (code == NULL)
    {
        gtk_combo_box_set_active (GTK_COMBO_BOX
            (dialog->screen_combo), 0);
        return;
    }

    int i = 1;
    while (resolutions[i] != '\0') {
        if (strcmp (code, get_first_word (resolutions[i])) == 0)
            gtk_combo_box_set_active (GTK_COMBO_BOX
                (dialog->screen_combo), i);
        i++;
    }
}

const gchar *get_screen ()
{
    const gchar *ret;
    gchar *tmp = get_first_word (gtk_combo_box_get_active_text (
        GTK_COMBO_BOX (dialog->screen_combo)));
    if (tmp) {
        ret = (const gchar *) g_strconcat (" screensize=", tmp, NULL);
        g_free (tmp);
    }
    else
        ret = "";
    return ret;
}

const gchar *get_screen_value ()
{
    const gchar *ret;
    gchar *tmp = get_first_word (gtk_combo_box_get_active_text (
        GTK_COMBO_BOX (dialog->screen_combo)));
    if (tmp) {
        ret = (const gchar *) tmp;
        g_free (tmp);
    }
    else
        ret = "";
    return ret;
}

/* show next time */
void set_next_time (gboolean nt)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->next_time), nt);
}

gboolean get_next_time ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->next_time));
}

/* no X */
void set_no_x (gboolean nox)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->nox_chk), nox);
}

const gchar *get_no_x ()
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->nox_chk)))
        return " noX";
    return "";
}

/* checkfs */
void set_checkfs (gboolean check)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkfs_chk), check);
}

const gchar *get_checkfs ()
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkfs_chk)))
        return " checkfs";
    return "";
}

/* persistence */
void set_persistent (const gchar *persistent)
{
    if (strncmp (persistent, "full", 4) == 0)
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->full_chk), TRUE);
    else
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->home_chk), TRUE);
}

const gchar *get_persistent ()
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->full_chk)))
        return " persistent=full";
    return "";
}

const gchar *get_persistent_value ()
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->full_chk)))
        return "full";
    return "";
}

/* lock active*/
void set_lock (gboolean locked)
{
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->lock_chk), locked);
    gtk_widget_set_sensitive (dialog->vbox_lock, locked);
}

gboolean get_lock ()
{
    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->lock_chk));
}

/* root device */
void set_root_device (const gchar *root)
{
    /* not editable by the user but we need to keep it somewher */
    g_object_set_data (G_OBJECT (dialog->dialog), "rootdevice",
        (gpointer) g_strdup (root));
}

const gchar *get_root_device ()
{
    gchar *rootdev = g_strdup (
        g_object_get_data (G_OBJECT (dialog->dialog), "rootdevice"));
    if (rootdev && strcmp (rootdev, "") != 0)
    {
        gchar *tmp = NULL;
        tmp = g_strconcat (" rootdevice=", rootdev, NULL);
        const gchar *ret = g_strdup (tmp);
        g_free (rootdev);
        g_free (tmp);
        return (ret);
    }
    g_free (rootdev);
    return "";
}

#ifdef EEE
/* lid */
void set_lid_action ()
{
    int id = LID_NONE;
    XfceRc *rc = xfce_rc_simple_open (ACPICONFFILE, TRUE);

    if (rc)
    {
        gchar *tmp = g_strdup (xfce_rc_read_entry (rc, "LID_ACTION", "none"));
        xfce_rc_close (rc);
        gchar *value = g_shell_unquote (tmp, NULL);
        g_free (tmp);
        if (strncmp (value, "suspend", 4) == 0)
            id = LID_SUSPEND;
        else if (strncmp (value, "halt", 4) == 0)
            id = LID_HALT;

        g_free (value);
    }
    gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->lid_combo), id);
}

int get_lid_action ()
{
    return gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->lid_combo));
}

void write_lid_action ()
{
    const gchar *value;
    int id = get_lid_action ();
    switch (id)
    {
        case LID_HALT:
            value = "halt";
            break;
        case LID_SUSPEND:
            value = "suspend";
            break;
        case LID_NONE:
        default:
            value = "none";
            break;
    }

    XfceRc *rc = xfce_rc_simple_open (ACPICONFFILE, FALSE);
    if (!rc)
        return;
    gchar *tmp = g_shell_quote (value);
    xfce_rc_write_entry (rc, "LID_ACTION", tmp);
    xfce_rc_close (rc);
    g_free (tmp);
}
#endif /* EEE */

/* reboot */
void set_reboot_needed (gboolean needed)
{
    g_object_set_data (G_OBJECT (dialog->dialog), "reboot",
        (gpointer) needed);
}

gboolean get_reboot_needed ()
{
    return (gboolean) g_object_get_data (G_OBJECT (dialog->dialog), "reboot");
}

/* update kbd and tz if needed */
void update_locale_settings (const gchar *locale)
{
    gboolean kbd_user = get_kbd_user_defined ();
    gboolean tz_user = get_tz_user_defined ();
    if (kbd_user && tz_user)
        return;

    gchar *kbd, *tz;
    get_from_language (locale, &kbd, &tz);

    if (!kbd_user)
    {
        set_kbd ((const gchar *)kbd);
        apply_kbd_config ((const gchar *)kbd);
    }
    if (!tz_user)
        set_tz ((const gchar *)tz);

    g_free (kbd);
    g_free (tz);
}


static gpointer get_initial_conf (int id)
{
    switch (id)
    {
        case INIT_SCREEN:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_screen");
            break;
        case INIT_PERSISTENT:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_persistent");
            break;
        case INIT_LOCK:
            return g_object_get_data (G_OBJECT (dialog->dialog),
                "init_lock");
            break;
    }
    return (gpointer) NULL;
}

static void set_initial_conf (gchar *screen, gchar *persistent,
                              gchar *kbd, gboolean lock)
{
    g_object_set_data (G_OBJECT (dialog->dialog), "init_screen",
        (gpointer) g_strdup (screen));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_persistent",
        (gpointer) g_strdup (persistent));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_kbd",
        (gpointer) g_strdup (kbd));
    g_object_set_data (G_OBJECT (dialog->dialog), "init_lock",
        (gpointer) lock);
}

static gboolean need_reboot ()
{
    const gchar *init, *now;
    gboolean linit, lnow;

    /* screen */
    init = (const gchar *) get_initial_conf (INIT_SCREEN);
    now = get_screen_value ();
    if (strcmp (init, now) != 0)
        return TRUE;

    /* persistence */
    init = (const gchar *) get_initial_conf (INIT_PERSISTENT);
    now = get_persistent_value ();
    if (strcmp (init, now) != 0)
        return TRUE;

    linit = (gboolean) get_initial_conf (INIT_LOCK);
    lnow = get_lock ();
    if (linit != lnow)
        return TRUE;

    return FALSE;
}

/* apply settings */
static void change_next_time()
{
    gchar dir[64];
    gchar cmd[128];
    gchar *tmp = NULL;
    gchar *home = NULL;
    uid_t uid = 1000;

    tmp = getenv("SUDO_UID");
    if (tmp)
        uid = atoi(tmp);

    home = get_home ();
    g_sprintf(dir, "%s/.config/autostart/", home);
    g_free (home);
    mkdir(dir, 0755);

    if (get_next_time ()) {
        g_sprintf(cmd, "cp %s %s", DESKTOPFILE, dir);
        system((const char *)cmd);
        g_sprintf(cmd, "%s/lconfeditor.desktop", dir);
        chown(dir, uid, uid);
    } else {
        g_sprintf(cmd, "%s/lconfeditor.desktop", dir);
        unlink(cmd);
    }
}

/**********************************/
void read_config ()
{
    FILE *fp;
    gchar nexttime_txt[64];

    gchar *tz_txt = g_strdup ("Europe/London");
    gchar *locale_txt = g_strdup ("en_US.UTF-8");
    gchar *kbd_code_txt = g_strdup ("us");
    gchar *screen_txt = g_strdup ("");
    gchar *persistent = g_strdup ("");
    gchar *rootdevice = g_strdup ("");
    gboolean nox = FALSE;
    gboolean checkfs = FALSE;
    gboolean nexttime = FALSE;
    gboolean lock = FALSE;

    dialog->lock_pwd = NULL;

    if (G_LIKELY((fp = fopen(CONFFILE, "r")))) {
        gchar *gline = NULL, *lline = NULL;
        gchar *tmp;
        size_t len = 0;
        ssize_t read;

        while ((read = getline(&tmp, &len, fp)) != -1) {
            if (!gline && (strncmp(tmp, "append  ", 8) == 0)) {
                g_free (gline);
                gline = g_strdup(tmp);
                if (gline[strlen(gline) - 1] == '\n')
                    gline[strlen(gline) - 1] = '\0';
            }
            if (!lline
                && (strncmp(tmp, "MENU MASTER PASSWD", 18) == 0)) {
                    lock = TRUE;
                    lline = g_strdup(tmp);
                    if (lline[strlen(lline) - 1] == '\n')
                        lline[strlen(lline) - 1] = '\0';
                    gchar **exploded = g_strsplit(lline, " ", 0);
                    if (g_strv_length(exploded) == 4)
                        dialog->lock_pwd =
                            g_strdup(exploded[3]);
                    g_strfreev(exploded);
                }
        }
        g_free(lline);
        g_free(tmp);
        fclose(fp);

        if (G_LIKELY(gline)) {
            gchar **exploded = g_strsplit(gline, " ", 0);
            int count = 0;
            while (exploded[count] != '\0') {
                gchar **item =
                    g_strsplit(exploded[count], "=", 0);
                if (g_strv_length(item) == 2) {
                    if (item[0] && item[1]
                        && strncmp(item[0], "xkbdlayout", 10) == 0)
                    {
                        g_free (kbd_code_txt);
                        kbd_code_txt = g_strdup (item[1]);
                    }
                    if (item[0] && item[1]
                        && strncmp(item[0], "locale", 6) == 0)
                    {
                        g_free (locale_txt);
                        locale_txt = g_strdup (item[1]);
                    }
                    if (item[0] && item[1]
                        && strncmp(item[0], "timezone", 8) == 0)
                    {
                        g_free (tz_txt);
                        tz_txt =g_strdup (item[1]);
                    }
                    if (item[0] && item[1]
                        && strncmp(item[0], "screensize", 10) == 0)
                    {
                        g_free (screen_txt);
                        screen_txt = g_strdup (item[1]);
                    }
                    if (item[0] && item[1]
                        && strncmp(item[0], "persistent", 10) == 0)
                    {
                        g_free (persistent);
                        persistent = g_strdup (item[1]);
                    }
                    if (item[0] && item[1]
                        && strncmp(item[0], "rootdevice", 10) == 0)
                    {
                        g_free (rootdevice);
                        rootdevice = g_strdup (item[1]);
                    }
                } else {
                    if (strncmp (exploded[count], "noX", 3) == 0)
                        nox = TRUE;
                    if (strncmp (exploded[count], "checkfs", 7) == 0)
                        checkfs = TRUE;
                    if (strncmp (exploded[count], "lockmode", 10) == 0) {
                        lock = TRUE;
                    }
                }
                g_strfreev(item);
                count++;
            }
            g_free(gline);
            g_strfreev(exploded);
        }
    }

    g_sprintf(nexttime_txt, "%s/.config/autostart/lconfeditor.desktop",
          get_home());
    if ((g_access("/cdrom/lconfeditor-stamp", R_OK) != 0)
        || (g_access(nexttime_txt, R_OK) == 0)) {
            nexttime = TRUE;
    }

    set_locale (locale_txt);
    set_kbd (kbd_code_txt);
    set_tz (tz_txt);
    set_screensize (screen_txt);
    set_next_time (nexttime);
    set_no_x (nox);
    set_checkfs (checkfs);
    set_persistent (persistent);
    set_root_device (rootdevice);
    set_lock (lock);
    set_initial_conf (screen_txt, persistent, kbd_code_txt, lock);
    set_reboot_needed (FALSE);

#ifdef EEE
    set_lid_action ();
#endif

    set_kbd_user_defined (FALSE);
    set_tz_user_defined (FALSE);

    g_free (tz_txt);
    g_free (locale_txt);
    g_free (kbd_code_txt);
    g_free (screen_txt);
    g_free (persistent);
    g_free (rootdevice);
}

void
write_config ()
{
    gchar *tz_txt = g_strdup (get_tz ());
    gchar *locale_txt = g_strdup (get_locale ());
    gchar *kbd_code_txt = g_strdup (get_kbd ());
    gchar *screen_txt = g_strdup (get_screen ());
    gchar *persistent = g_strdup (get_persistent ());
    gchar *rootdevice = g_strdup (get_root_device ());
    gchar *nox = g_strdup (get_no_x ());
    gchar *checkfs = g_strdup (get_checkfs ());
    gboolean lock = get_lock ();
    gchar *str = NULL;
    gchar output[BUFSIZ];
    FILE *f;

    str = g_strconcat (
        tz_txt,
        locale_txt,
        kbd_code_txt,
        screen_txt,
        persistent,
        rootdevice,
        nox,
        checkfs,
        NULL);

    if (lock)
        g_sprintf (output, locked_syscfg, dialog->lock_pwd, str);
    else
        g_sprintf (output, default_syscfg, str);
    if ((f = fopen (CONFFILE, "w"))) {
        fputs (output, f);
        fclose (f);
        if (G_LIKELY ((f = fopen ("/cdrom/lconfeditor-stamp", "w"))))
            fclose (f);
    }

    /* apply other settings */
    change_next_time ();
    mount_save ();
    run_user_cmd ("linutop-startup --samba", FALSE);
    write_lock_config ();
#ifdef EEE
    write_lid_action ();
#endif
    if (need_reboot ())
        set_reboot_needed (TRUE);

    g_free (tz_txt);
    g_free (locale_txt);
    g_free (kbd_code_txt);
    g_free (screen_txt);
    g_free (persistent);
    g_free (rootdevice);
    g_free (nox);
    g_free (checkfs);
    g_free (str);
}
