/* main.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"

gboolean have_to_reboot = FALSE;

/* callbacks */
static void run_help(GtkWidget * widget, gpointer data)
{
	run_user_cmd("mousepad /cdrom/readme.txt", TRUE);
}
static void delete_event(GtkWidget * widget, GdkEvent * event, gpointer data)
{
	/* restore the initial kbd conf */
    restore_kbd_config ();
	gtk_main_quit();
}

static void kbd_btn_clicked(GtkWidget * widget, gpointer data)
{
	layout_dialog_run ();
}

static void tz_btn_clicked(GtkWidget * widget, gpointer data)
{
	tz_dialog_run ();
}

static void locale_btn_clicked(GtkWidget * widget, gpointer data)
{
	locale_dialog_run ();
}

static void change_pwd (GtkWidget * w, gpointer data)
{
	gchar *tmp = get_lock_passwd ();
	if (tmp)
    {
        g_free (dialog->lock_pwd);
		dialog->lock_pwd = tmp;
    }
}

static void lock_toggled (GtkWidget * btn, gpointer data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (btn))) {
		if (!dialog->lock_pwd) {
			gchar *tmp = get_lock_passwd();
			if (tmp) {
				gtk_widget_set_sensitive(dialog->vbox_lock,
							 TRUE);
                g_free (dialog->lock_pwd);
				dialog->lock_pwd = tmp;
			} else {
				gtk_widget_set_sensitive (dialog->vbox_lock,
							 FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
							     (btn), FALSE);
			}
		} else
			gtk_widget_set_sensitive(dialog->vbox_lock, TRUE);
	} else
		gtk_widget_set_sensitive(dialog->vbox_lock, FALSE);
}

static void save_config(GtkWidget * widget, gpointer data)
{
    const gchar *timezone = get_tz_value ();
    char *tmp =
        g_strconcat ("sh ", LIBEXECDIR, "/apply-tz.sh ", timezone, NULL);
    const gchar *command = g_strdup (tmp);
    g_free (tmp);

    write_config ();
    system (command);

    if (get_reboot_needed ())
    {
	    /* destroy the main window */
    	gtk_widget_destroy(dialog->dialog);
        have_to_reboot = warn_about_changes();
    }

	gtk_main_quit();
}

static void populate_screen_combo(GtkWidget * combo)
{
	int i = 0;
	while (resolutions[i] != '\0') {
		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), resolutions[i]);
		i++;
	}
}

static void create_dialog()
{
	GtkWidget *label, *table, *vbox, *hbox, *notebook, *vbox2;
	GtkWidget *asrun_btn, *backup_btn, *format_btn, *image, *frame;
    GtkWidget *locale_btn, *kbd_btn, *tz_btn, *net_btn, *lock_pwd_btn;
	GtkWidget *advanced;
	GtkWidget *mnt_hbox;
	GtkWidget *lock_w, *hr;
    GtkWidget *aboutlabel;
	GtkTooltips *tooltip;
#ifdef EEE
    GtkWidget *energy;
#endif

	gtk_window_set_default_icon_name("preferences-desktop");

	dialog->dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog->dialog), _("Linutop Setup"));
	gtk_window_set_resizable(GTK_WINDOW(dialog->dialog), FALSE);
	gtk_window_set_position(GTK_WINDOW(dialog->dialog), GTK_WIN_POS_CENTER);

	notebook = gtk_notebook_new();
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog->dialog)->vbox),
			  notebook);
	gtk_widget_show(notebook);

	/*
	 * General tab
	 */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);

	label = gtk_label_new(_("General"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);

	table = gtk_table_new(3, 3, FALSE);
	gtk_widget_show(table);

	/* local settings */
	frame = xfce_framebox_new(_("Local settings:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), table);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 4);
	gtk_widget_show(frame);

	label = gtk_label_new(_("Language: "));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL,
			 GTK_SHRINK, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_widget_show(label);
	dialog->locale_entry = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(dialog->locale_entry), FALSE);
	gtk_table_attach(GTK_TABLE(table), dialog->locale_entry, 1, 2, 0, 1,
			 GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
	gtk_widget_show(dialog->locale_entry);
	locale_btn = gtk_button_new();
	gtk_table_attach(GTK_TABLE(table), locale_btn, 2, 3, 0, 1,
			 GTK_SHRINK, GTK_SHRINK, 0, 0);
	image = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
	gtk_container_add(GTK_CONTAINER(locale_btn), image);
	gtk_widget_show(image);
	tooltip = gtk_tooltips_new();
	gtk_tooltips_set_tip(tooltip, locale_btn,
			     _("Clic here to select your language"), "");
	gtk_widget_show(locale_btn);

	label = gtk_label_new(_("Keyboard layout: "));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL,
			 GTK_SHRINK, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_widget_show(label);
	dialog->kbd_entry = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(dialog->kbd_entry), FALSE);
	gtk_table_attach(GTK_TABLE(table), dialog->kbd_entry, 1, 2, 1, 2,
			 GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
	gtk_widget_show(dialog->kbd_entry);
	kbd_btn = gtk_button_new();
	gtk_table_attach(GTK_TABLE(table), kbd_btn, 2, 3, 1, 2,
			 GTK_SHRINK, GTK_SHRINK, 0, 0);
	image = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
	gtk_container_add(GTK_CONTAINER(kbd_btn), image);
	gtk_widget_show(image);
	tooltip = gtk_tooltips_new();
	gtk_tooltips_set_tip(tooltip, kbd_btn,
			     _("Clic here to select your keyboard layout"), "");
	gtk_widget_show(kbd_btn);

	label = gtk_label_new(_("Timezone: "));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, GTK_FILL,
			 GTK_SHRINK, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_widget_show(label);
	dialog->tz_entry = gtk_entry_new();
	gtk_editable_set_editable(GTK_EDITABLE(dialog->tz_entry), FALSE);
	gtk_table_attach(GTK_TABLE(table), dialog->tz_entry, 1, 2, 2, 3,
			 GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
	gtk_widget_show(dialog->tz_entry);
	tz_btn = gtk_button_new();
	gtk_table_attach(GTK_TABLE(table), tz_btn, 2, 3, 2, 3,
			 GTK_SHRINK, GTK_SHRINK, 0, 0);
	image = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
	gtk_container_add(GTK_CONTAINER(tz_btn), image);
	gtk_widget_show(image);
	tooltip = gtk_tooltips_new();
	gtk_tooltips_set_tip(tooltip, tz_btn,
			     _("Clic here to select your time zone"), "");
	gtk_widget_show(tz_btn);

	/* Display settings */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new(_("Screen size:"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_widget_show(label);

	dialog->screen_combo = gtk_combo_box_new_text();
	gtk_box_pack_start(GTK_BOX(hbox), dialog->screen_combo, TRUE, TRUE, 2);
	populate_screen_combo(dialog->screen_combo);
	gtk_combo_box_set_active(GTK_COMBO_BOX(dialog->screen_combo), 0);
	gtk_widget_show(dialog->screen_combo);

	frame = xfce_framebox_new(_("Display settings:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), hbox);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	/* Autostarted app */
	vbox2 = gtk_vbox_new(FALSE, 2);
	gtk_widget_show(vbox2);

	asrun_btn = new_run_cmd_button ("xfce4-autostart-editor",
                                    _("Configure autostarted applications"),
                                    "appstarter", TRUE);
	gtk_widget_show(asrun_btn);

	gtk_box_pack_start(GTK_BOX(vbox2), asrun_btn, FALSE, FALSE, 2);
	gtk_widget_show(asrun_btn);

	dialog->next_time =
	    gtk_check_button_new_with_label(_("Show this dialog at each boot"));
	gtk_box_pack_start(GTK_BOX(vbox2), dialog->next_time, FALSE, FALSE, 2);
	gtk_widget_show(dialog->next_time);

	frame = xfce_framebox_new(_("Autostarted application:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox2);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), _
			     ("<span style=\"italic\" size=\"small\">most changes will be effective after reboot.</span>"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 4);
	gtk_widget_show(label);

	/*
	 * Network tab
	 */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);

	label = gtk_label_new(_("Network"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);

	/* Network settings */
	net_btn = new_run_cmd_button ("network", _("Configure your network"),
                                  "network-admin", FALSE);
	gtk_widget_show(net_btn);

	frame = xfce_framebox_new(_("Network settings:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), net_btn);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	/* samba */
	mnt_hbox = create_mount_gui();
	frame = xfce_framebox_new(_("Windows shares:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), mnt_hbox);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	/*
	 * advanced tab
	 */
	advanced = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(advanced);

	/* lbackup */
	backup_btn = new_run_cmd_button ("document-save", _("Backup/Restore"),
                                     "lbackup", FALSE);
	gtk_widget_show(backup_btn);

	frame = xfce_framebox_new(_("Backups"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), backup_btn);
	gtk_box_pack_start(GTK_BOX(advanced), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	/* persistence */
	vbox2 = gtk_vbox_new(FALSE, 2);
	gtk_widget_show(vbox2);

	dialog->home_chk =
	    gtk_radio_button_new_with_label(NULL,
					    (_
					     ("user settings only persistence")));
	gtk_box_pack_start(GTK_BOX(vbox2), dialog->home_chk, FALSE, FALSE, 0);
	gtk_widget_show(dialog->home_chk);
	dialog->full_chk =
	    gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON
							(dialog->home_chk),
							_
							("full persistent system"));
	gtk_box_pack_start(GTK_BOX(vbox2), dialog->full_chk, FALSE, FALSE, 0);
	gtk_widget_show(dialog->full_chk);

	frame = xfce_framebox_new(_("Data persistence:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox2);
	gtk_box_pack_start(GTK_BOX(advanced), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	/* misc */
	vbox2 = gtk_vbox_new(FALSE, 2);
	gtk_widget_show(vbox2);

	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label),
			     _
			     ("<span style=\"italic\" size=\"small\">WARNING: Be sure to know "
			      "what you do before changing these settings.</span>"));
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 4);
	gtk_widget_show(label);

	dialog->nox_chk =
	    gtk_check_button_new_with_label(_
					    ("Don't start Xorg (command line system)"));
	gtk_box_pack_start(GTK_BOX(vbox2), dialog->nox_chk, FALSE, FALSE, 0);
	gtk_widget_show(dialog->nox_chk);

	dialog->checkfs_chk =
	    gtk_check_button_new_with_label(_
					    ("Check and try to fix filesystem errors"));
	gtk_box_pack_start(GTK_BOX(vbox2), dialog->checkfs_chk, FALSE, FALSE,
			   0);
	gtk_widget_show(dialog->checkfs_chk);

    /* usb cleanup */
    format_btn = new_run_cmd_button ("edit-clear", _("Format a USB key (FAT32)"),
                                     "lusbclean", FALSE);
    gtk_box_pack_start(GTK_BOX(vbox2), format_btn, FALSE, FALSE, 2);
	gtk_widget_show(format_btn);

	frame = xfce_framebox_new(_("Misc:"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox2);
	gtk_box_pack_start(GTK_BOX(advanced), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	/*
	 * soft lock tab
	 */
	label = gtk_label_new(_("Advanced"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), advanced, label);

	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox2);

	dialog->lock_chk =
	    gtk_check_button_new_with_label(_("Use the Linutop lock"));
	gtk_box_pack_start(GTK_BOX(vbox2), dialog->lock_chk, FALSE, FALSE, 4);
	gtk_widget_show(dialog->lock_chk);

	hr = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox2), hr, FALSE, FALSE, 2);
	gtk_widget_show(hr);

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 2);
	gtk_widget_show(hbox);

	image = gtk_image_new_from_icon_name("linutop-lock",
					     GTK_ICON_SIZE_DIALOG);
	gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 2);
	gtk_widget_show(image);

	label = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(label),
			     _("<span size=\"small\">"
			       "In locked mode the system cannot be altered\n(convenient for public access).\n\n"
			       "When switching to this mode, you will be prompted a password for the "
			       "admin user (called 'linutop'). This user only can switch back to the "
			       "standard mode. "
			       "To change the login, use the 'logout' button in the quit dialog.\n\n"
			       "To configure the unprivileged user, log in as 'user' in persistant mode. "
			       "Regular persistent mode allows you to add softwares.\n\n"
			       "Don't forget your password!" "</span>"));
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
	gtk_widget_show(label);

	hr = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox2), hr, FALSE, FALSE, 2);
	gtk_widget_show(hr);

	dialog->vbox_lock = gtk_vbox_new(FALSE, 2);
	gtk_widget_set_sensitive(dialog->vbox_lock, FALSE);
	gtk_widget_show(dialog->vbox_lock);

	lock_pwd_btn = gtk_button_new_with_label(_("Change password"));
	gtk_box_pack_start(GTK_BOX(dialog->vbox_lock), lock_pwd_btn,
			   FALSE, FALSE, 2);
	gtk_widget_show(lock_pwd_btn);

	lock_w = lock_widget();
	gtk_box_pack_start(GTK_BOX(dialog->vbox_lock), lock_w, FALSE, FALSE, 2);
	gtk_widget_show(lock_w);

	frame = xfce_framebox_new(_("Settings"), TRUE);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
	xfce_framebox_add(XFCE_FRAMEBOX(frame), dialog->vbox_lock);
	gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 10);
	gtk_widget_show(frame);

	label = gtk_label_new(_("Security"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox2, label);

#ifdef EEE
    /*
     * Energy
     */
    energy = get_energy_widget ();
    gtk_widget_show (energy);

    label = gtk_label_new(_("Energy"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), energy, label);
#endif /* EEE */

    /* About */
    gchar *abouttext = g_strconcat (
        "<span size=\"large\" weight=\"bold\">"
        "Linutop Setup Menu v. " PACKAGE_VERSION
        "</span>\n\n",
        _("Designed for quick and simple setup "
          "of your Linutop computer.\n\n"
          "More on http://www.linutop.com/wiki"),
        NULL);

    aboutlabel = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (aboutlabel), abouttext);
    gtk_label_set_justify (GTK_LABEL (aboutlabel), GTK_JUSTIFY_CENTER);
    gtk_widget_show (aboutlabel);
    g_free (abouttext);

    label = gtk_label_new(_("About"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), aboutlabel, label);

    /* buttons */
    hbox = GTK_DIALOG (dialog->dialog)->action_area;

	dialog->help_btn = gtk_button_new_from_stock (GTK_STOCK_HELP);
	gtk_box_pack_start (GTK_BOX (hbox), dialog->help_btn, TRUE, FALSE, 0);
    gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (hbox),
                                        dialog->help_btn, TRUE);
	gtk_widget_set_sensitive (dialog->help_btn,
				 (g_access (README, R_OK) == 0));
	gtk_widget_show (dialog->help_btn);

	dialog->quit_btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
	gtk_box_pack_start (GTK_BOX (hbox),dialog->quit_btn, TRUE, FALSE, 0);
	gtk_widget_show (dialog->quit_btn);

	dialog->save_btn = gtk_button_new_from_stock (GTK_STOCK_OK);
	gtk_box_pack_start (GTK_BOX (hbox), dialog->save_btn, TRUE, FALSE, 0);
	gtk_widget_set_sensitive (dialog->save_btn,
				 (g_access (CONFFILE, W_OK) == 0));
	gtk_widget_show (dialog->save_btn);

	read_config ();

	/* signals */
	g_signal_connect(G_OBJECT(kbd_btn), "clicked",
			 G_CALLBACK(kbd_btn_clicked), NULL);
	g_signal_connect(G_OBJECT(tz_btn), "clicked",
			 G_CALLBACK(tz_btn_clicked), NULL);
	g_signal_connect(G_OBJECT(locale_btn), "clicked",
			 G_CALLBACK(locale_btn_clicked), NULL);

	g_signal_connect(G_OBJECT(dialog->lock_chk), "toggled",
			 G_CALLBACK(lock_toggled), NULL);
	g_signal_connect(G_OBJECT(lock_pwd_btn), "clicked",
			 G_CALLBACK(change_pwd), NULL);

	g_signal_connect(G_OBJECT(dialog->help_btn), "clicked",
			 G_CALLBACK(run_help), NULL);
	g_signal_connect(G_OBJECT(dialog->save_btn), "clicked",
			 G_CALLBACK(save_config), NULL);
	g_signal_connect(G_OBJECT(dialog->quit_btn), "clicked",
			 G_CALLBACK(delete_event), NULL);
	g_signal_connect(G_OBJECT(dialog->dialog), "delete_event",
			 G_CALLBACK(delete_event), NULL);

	gtk_widget_show(dialog->dialog);
}

int main(int argc, char *argv[])
{
	xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
	gtk_init(&argc, &argv);
	gdk_notify_startup_complete();
	int id = getuid();
	if (id != 0) {
		xfce_warn(_("You don't appear to be root. Exiting."));
		exit(1);
	}

	dsp = XOpenDisplay(NULL);
	g_type_init();
	engine = xkl_engine_get_instance(dsp);
	registry = xkl_config_registry_get_instance(engine);
	xkl_config_registry_load(registry);

	dialog = g_new0(gui, 1);
	dialog->lock_pwd = NULL;

	create_dialog();

	gtk_main();

	g_free(dialog);

	if (have_to_reboot)
		system(REBOOTCMD);

	return 0;
}
