/* language.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <gtk/gtk.h>
#include "utils.h"

/* keyboard globals */
gchar *l = NULL;
gchar *v = NULL;

/* */
const gchar *resolutions[11] = {
   "Auto",
   "640x480 (15'')",
   "800x600 (15'')",
   "1024x768 (15'')",
   "1280x1024 (17-19'')",
   "1280x768 (17'' wide)",
   "1366x768 (19'' wide)",
   "1600x1200 (20'' wide)",
   "1680x1050 (22'' wide)",
   "1920x1440 (Max)",
   NULL
};

/* language data */
const struct locale_item locale_data[] = {
    { "Arabic", "ar_EG.UTF-8", "ara", "Africa/Cairo" },
    { "Czech", "cs_CZ.UTF-8", "cz", "Europe/Prague" },
    { "Danish", "da_DK.UTF-8", "dk", "Europe/Copenhagen" },
    { "Dutch", "nl_NL.UTF-8", "nl", "Europe/Amsterdam" },
    { "English", "en_US.UTF-8", "us", "Europe/London" },
    { "Finnish", "fi_FI.UTF-8", "fi", "Europe/Helsinki" },
    { "French", "fr_FR.UTF-8", "fr", "Europe/Paris" },
    { "German", "de_DE.UTF-8", "de", "Europe/Berlin" },
    { "Greek", "el_GR.UTF-8", "gr", "Europe/Athens" },
    { "Hungarian", "hu_HU.UTF-8", "hu", "Europe/Budapest" },
    { "Italian", "it_IT.UTF-8", "it", "Europe/Rome" },
    { "Japanese", "ja_JP.UTF-8", "jp", "Asia/Tokyo" },
    { "Norwegian Bokmaal", "nb_NO.UTF-8", "no", "Europe/Oslo" },
    { "Polish", "pl_PL.UTF-8", "pl", "Europe/Warsaw" },
    { "Portuguese", "pt_PT.UTF-8", "pt", "Europe/Lisbon" },
    { "Portuguese (Brazil)", "pt_BR.UTF-8", "pt", "Brazil/East" },
    { "Romanian", "ro_RO.UTF-8", "ro", "Europe/Bucharest" },
    { "Russian", "ru_RU.UTF-8", "ru", "Europe/Moscow" },
    { "Slovak", "sk_SK.UTF-8", "sk", "Europe/Bratislava" },
    { "Slovenian", "sl_SI.UTF-8", "si", "Europe/Ljubljana" },
    { "Spanish", "es_ES.UTF-8", "es", "Europe/Madrid" },
    { NULL, NULL, NULL, NULL }
};

/* GTK utils */
void cb_row_activated(GtkWidget * dialog, gpointer data)
{
  gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
}

static void
run_btn_clicked (GtkWidget *btn, gpointer data)
{
    gchar *cmd = NULL;
    gboolean as_user;

    cmd = (gchar *) g_object_get_data (G_OBJECT (btn), "app");
    as_user = (gboolean) g_object_get_data (G_OBJECT (btn), "as_user");

    if (as_user)
        run_user_cmd (cmd, TRUE);
    else
        xfce_exec (cmd, FALSE, TRUE, NULL);
}

GtkWidget *
new_run_cmd_button (const gchar *icon, const gchar *txt,
                    const gchar *app, gboolean as_user)
{
    GtkWidget *btn, *hbox, *label, *image;

    hbox = gtk_hbox_new(FALSE, 2);

    image = gtk_image_new_from_icon_name (icon, GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 2);

    label = gtk_label_new (txt);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

    gtk_widget_show_all (hbox);

    btn = gtk_button_new ();
    g_object_set_data (G_OBJECT (btn), "app", (gpointer) app);
    g_object_set_data (G_OBJECT (btn), "as_user", (gpointer) as_user);
    gtk_widget_set_sensitive(btn, app_is_usable (app));
    gtk_container_add(GTK_CONTAINER(btn), hbox);

    g_signal_connect (G_OBJECT (btn), "clicked",
        G_CALLBACK (run_btn_clicked), NULL);

    return btn;
}

/* Text utils */
gboolean app_is_usable (const gchar *app)
{
    gboolean ret = FALSE;
    gchar *fullname = g_find_program_in_path (app);
    if (fullname && g_access (fullname, X_OK) == 0)
        ret = TRUE;
    g_free (fullname);
    return ret;
}

gchar *get_home()
{
    struct passwd *pw;
    uid_t uid = 0;
    gchar *tmp = NULL;
    gchar *ret = NULL;

    tmp = getenv("SUDO_UID");
    if (tmp)
        uid = atoi(tmp);
    if (uid >= 0)
        pw = getpwuid(uid);
    if (pw)
        ret = g_strdup (pw->pw_dir);
    else
        ret = g_strdup ("root");

    return ret;
}

gchar *get_first_word(const gchar *line)
{
    gchar **tab;
    gchar *word;
    const char *del = " ";
    if (strncmp(line, "Auto", 4) == 0)
        return NULL;
    tab = g_strsplit(line, del, 2);
    if (tab[0] != '\0') {
        word = g_strdup(tab[0]);
        g_strfreev(tab);
        return word;
    }
    return NULL;
}

void run_user_cmd(const char *user_cmd, gboolean sn)
{
    uid_t uid = 0;
    char *cuid = NULL;
    gchar cmd[100];
    gchar *run_sn = sn ? "" : "-n";

    cuid = getenv("SUDO_UID");
    if (cuid)
        uid = atoi(cuid);
    if (uid >= 0) {
        g_sprintf(cmd, "lstart %s -c \"%s\" -u %d", run_sn, user_cmd,
              uid);
        xfce_exec(cmd, FALSE, FALSE, NULL);
    }
}

gchar *get_locale_name (const gchar *code)
{
    gchar *name = NULL;
    int count = 0;
    while (locale_data[count].code != '\0') {
        if (!strncmp(locale_data[count].code, code, strlen(code))) {
            name = g_strdup (locale_data[count].name);
            return name;
        }
        count++;
    }
    return name;
}

/* kbd utils */
void apply_kbd_config (const gchar *kbd)
{
    gchar cmd[100];
    g_sprintf(cmd, "setxkbmap %s", kbd);
    xfce_exec(cmd, FALSE, FALSE, NULL);
}

static void
get_variant_desc (XklConfigRegistry * config_registry,
                  XklConfigItem * config_item, gchar ** explode)
{
    if (strcmp (explode[1], config_item->name) == 0)
        v = g_strdup (config_item->description);
}

static void
get_layout_desc (XklConfigRegistry * config_registry,
        XklConfigItem * config_item, gchar ** explode)
{
    if (strcmp (explode[0], config_item->name) == 0) {
        l = g_strdup (config_item->description);
        if (explode[1]) {
            xkl_config_registry_foreach_layout_variant
                (config_registry, config_item->name,
                 (ConfigItemProcessFunc)
                 get_variant_desc, explode);
        }
    }
}

gchar *get_kbd_name (const gchar *str)
{
    gchar **explode;
    gchar *ret;
    explode = g_strsplit (str, "_", 0);

    xkl_config_registry_foreach_layout (registry, (ConfigItemProcessFunc)
                       get_layout_desc, explode);

    if (explode[1])
        ret = g_strconcat (l, " (", v, ")", NULL);
    else
        ret = g_strdup (l);

    g_free (l);
    g_free (v);
    l = NULL;
    v = NULL;
    g_strfreev (explode);

    return ret;
}

void get_from_language (const gchar *locale, gchar **kbd, gchar **tz)
{
    int i;
    gboolean found = FALSE;
    for (i=0; locale_data[i].code != NULL; i++)
    {
        if (strncmp (locale, locale_data[i].code, strlen (locale)) == 0)
        {
            *kbd = g_strdup (locale_data[i].kbd);
            *tz = g_strdup (locale_data[i].tz);
            found = TRUE;
            break;
        }
    }
    /* we shouldn't get here */
    if (!found)
    {
        *kbd = NULL;
        *tz = NULL;
    }
}

